#	Masrv
#	Copyright (C) 2007 Michal Vaner (vorner@ucw.cz)
#
#	This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	he Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import glob
import config
import os
import os.path
import popen2
import splitter
import fcntl
import sys
import re
import socket
import datetime
import modeler
import hooks
import selector
from sender import sender

class source( sender ):
	def __init__( self, images ):
		sender.__init__( self )
		glob.info( "Starting the source plugin" )

		if not "source" in config.data:
			config.fatal( "No source plugin set" )

		conffile = os.path.abspath( config.data[ "file" ] )

		if "dir" in config.data:
			os.chdir( config.data[ "dir" ] )

		( serv, plug ) = socket.socketpair()
		self.__pid = os.fork()
		if not self.__pid:
			serv.close()
			os.dup2( plug.fileno(), sys.stdin.fileno() )
			os.dup2( plug.fileno(), sys.stdout.fileno() )
			os.dup2( plug.fileno(), sys.stderr.fileno() )
			plug.close()

			os.execlp( config.data[ "source" ], config.data[ "source" ] )

		plug.close()
		self.__firstReg = re.compile( "^(.)(\[(.*)\])?:(.*)$" )
		self.__modeler = modeler.modeler( self, images )
		self.__hooks = hooks.hooks( self, self.__modeler )
		self.__handlers = { "F":self.__errorHandler, "Q":self.__queryHandler, "D":self.__debugHandler, "M":self.__modeler.command, "H":self.__hooks.command }
		self.__queries = { "time":self.__timeQuery, "timestamp":self.__tstampQuery, "weekday":self.__weekday, "cd":self.__cdQuery, "imgsize":self.__imgsize }
		self.__commandReg = re.compile( "^([^ ]*)\s*(.*\S)?\s*$" )
		self.__sock = serv
		self.__splitter = splitter.splitter( self )

		self.send( "I:dimensions " + str( config.data.get( "width", 2048 ) ) + " " + str( config.data.get( "height", 768 ) ) + "\nI:config \"" + conffile + "\"\n" )
		selector.insertPeriodic( self.__hooks )
		self.__images = images

	def fileno( self ):
		return self.__sock.fileno()

	def read( self ):
		msg = self.__sock.recv( 4096 )
		if len( msg ) == 0:
			#It died? What now? Panic...
			glob.fatal( "Lost source plugin" )
		else:
			#glob.info( "Source data " + msg )
			self.__splitter.insert( msg )

	def err( self ):
		glob.fatal( "Lost source plugin" )

	def reportLine( self, line ):
		if( line == "" ):
			return
		mo = self.__firstReg.match( line )
		if not mo:
			glob.fatal( "Invalid syntax from source plugin " + line )
		if not mo.group( 1 ) in self.__handlers:
			glob.fatal( "Unknown command from source plugin " + line )
		self.__handlers[ mo.group( 1 ) ]( mo.group( 3 ), mo.group( 4 ) )

	def send( self, msg ):
		#glob.info( "Sending to source plugin: " + msg )
		self.enque( msg )

	def sockSend( self, data ):
		return self.__sock.send( data )

	def __errorHandler( self, id, command ):
		glob.fatal( "Source plugin: " + command )

	def __debugHandler( self, id, command ):
		glob.info( "Debug info from plugin: " + command )

	def __queryHandler( self, id, command ):
		glob.info( "Query" )
		mo = self.__commandReg.match( command )
		if not mo:
			glob.fatal( "Invalid query syntax " + command )
		if not mo.group( 1 ) in self.__queries:
			glob.fatal( "Unknown query " + mo.group( 1 ) )
		self.__queries[ mo.group( 1 ) ]( id, mo.group( 2 ) )

	def __timeQuery( self, id, args ):
		glob.info( "Sending time" )
		self.send( "R[" + id + "]:time " + datetime.datetime.now().isoformat( ' ' )  + "\n" )

	def __tstampQuery( self, id, args ):
		glob.info( "Sending timestamp" )
		self.send( "R[" + id + "]:timestamp " + str( glob.timestamp() ) + "\n" )

	def __weekday( self, id, args ):
		glob.info( "Sending weekday" )
		self.send( "R[" + id + "]:weekday " + str( datetime.datetime.now().weekday() ) + "\n" )

	def __cdQuery( self, id, args ):
		glob.info( "Changing directory" )
		os.chdir( args )
		self.send( "R[" + id + "]:cd\n" )

	def __imgsize( self, id, args ):
		glob.info( "Request for image size" )
		( x, y ) = self.__images.size( args )
		self.send( "R[" + id + "]:imgsize " + str( x ) + " " + str( y ) + "\n" )

	def modeler( self ):
		return self.__modeler
